From ac2f687c45c6a298fbc1cbfd6b7b21f5f17c3d3c Mon Sep 17 00:00:00 2001
From: Stefan Kangas <stefan@marxist.se>
Date: Sun, 3 Oct 2021 00:41:13 +0200
Subject: [PATCH] Make third mapconcat argument optional

* src/fns.c (Fmapconcat): Make 'separator' argument optional.  (Bug#50965)
* test/src/fns-tests.el (fns-tests-mapconcat): New test.
* lisp/emacs-lisp/subr-x.el (string-join): Document 'separator'
argument better.
---
 etc/NEWS                  | 3 +++
 lisp/emacs-lisp/subr-x.el | 4 +++-
 src/fns.c                 | 8 ++++++--
 test/src/fns-tests.el     | 7 +++++++
 4 files changed, 19 insertions(+), 3 deletions(-)

diff --git a/etc/NEWS b/etc/NEWS
index 8c22230daf..7be043fc99 100644
--- a/etc/NEWS
+++ b/etc/NEWS
@@ -80,6 +80,9 @@ Emacs buffers, like indentation and the like.  The new ert function
 
 * Lisp Changes in Emacs 29.1
 
+** Third 'mapconcat' argument 'separator' is now optional.
+An explicit nil always meant the empty string, now it can be left out.
+
 
 * Changes in Emacs 29.1 on Non-Free Operating Systems
 
diff --git a/lisp/emacs-lisp/subr-x.el b/lisp/emacs-lisp/subr-x.el
index 91ebbf9fb9..2bdbdaab74 100644
--- a/lisp/emacs-lisp/subr-x.el
+++ b/lisp/emacs-lisp/subr-x.el
@@ -208,7 +208,9 @@ string-empty-p
   (string= string ""))
 
 (defsubst string-join (strings &optional separator)
-  "Join all STRINGS using SEPARATOR."
+  "Join all STRINGS using SEPARATOR.
+Optional argument SEPARATOR must be a string, a vector, or a list of
+characters; nil is the empty string."
   (mapconcat #'identity strings separator))
 
 (define-obsolete-function-alias 'string-reverse 'reverse "25.1")
diff --git a/src/fns.c b/src/fns.c
index a72e41aee5..f77b6009bd 100644
--- a/src/fns.c
+++ b/src/fns.c
@@ -2852,12 +2852,16 @@ mapcar1 (EMACS_INT leni, Lisp_Object *vals, Lisp_Object fn, Lisp_Object seq)
   return leni;
 }
 
-DEFUN ("mapconcat", Fmapconcat, Smapconcat, 3, 3, 0,
+DEFUN ("mapconcat", Fmapconcat, Smapconcat, 2, 3, 0,
        doc: /* Apply FUNCTION to each element of SEQUENCE, and concat the results as strings.
 In between each pair of results, stick in SEPARATOR.  Thus, " " as
   SEPARATOR results in spaces between the values returned by FUNCTION.
+
 SEQUENCE may be a list, a vector, a bool-vector, or a string.
-SEPARATOR must be a string, a vector, or a list of characters.
+
+Optional argument SEPARATOR must be a string, a vector, or a list of
+characters; nil is the empty string.
+
 FUNCTION must be a function of one argument, and must return a value
   that is a sequence of characters: either a string, or a vector or
   list of numbers that are valid character codepoints.  */)
diff --git a/test/src/fns-tests.el b/test/src/fns-tests.el
index 5759457209..a771790b68 100644
--- a/test/src/fns-tests.el
+++ b/test/src/fns-tests.el
@@ -430,6 +430,13 @@ fns-tests-hash-buffer
                    (buffer-hash))
                  (sha1 "foo"))))
 
+(ert-deftest fns-tests-mapconcat ()
+  (should (string= (mapconcat #'identity '() "_") ""))
+  (should (string= (mapconcat #'identity '("A") "_") "A"))
+  (should (string= (mapconcat #'identity '("A" "B") "_") "A_B"))
+  (should (string= (mapconcat #'identity '("A" "B" "C") "_") "A_B_C"))
+  (should (string= (mapconcat #'identity '("a" "b")) "ab")))
+
 (ert-deftest fns-tests-mapcan ()
   (should-error (mapcan))
   (should-error (mapcan #'identity))
-- 
2.33.0

